#!/usr/bin/env python

import unittest
import doctest
import sys
import os


class SurfaceStub(object):

    w = 800
    h = 600

    def get_size(self):
        return (self.w, self.h)


def doctest_Viewport():
    """Tests for Viewport

        >>> from world import Vector
        >>> from ui import Viewport
        >>> viewport = Viewport(SurfaceStub())
        >>> viewport.origin
        Vector(0, 0)
        >>> viewport.scale
        1.0

    To make things more interesting we will set a scale of 0.2 and shift
    the origin to (100, 200)

        >>> viewport.scale = 0.2
        >>> viewport.origin = Vector(100, 200)

    ``screen_len`` converts a length in world units to screen units

        >>> viewport.screen_len(50)
        10
        >>> viewport.screen_len(6)
        1

    ``screen_pos`` converts world coordinates to screen coordinates

        >>> viewport.screen_pos(Vector(100, 200))
        (400, 300)
        >>> viewport.screen_pos(Vector(150, 250))
        (410, 290)
        >>> viewport.screen_pos(Vector(106, 199))
        (401, 300)

    ``world_pos`` converts screen coordinates to world coordinates

        >>> viewport.world_pos((400, 300))
        (100.0, 200.0)
        >>> viewport.world_pos((410, 290))
        (150.0, 250.0)
        >>> viewport.world_pos((401, 300))
        (105.0, 200.0)

    """


def doctest_Viewport_screen_size_change():
    """Tests for Viewport

        >>> from world import Vector
        >>> from ui import Viewport
        >>> viewport = Viewport(SurfaceStub())

    ``screen_pos`` converts world coordinates to screen coordinates

        >>> viewport.screen_pos(Vector(0, 0))
        (400, 300)

    You can change the size of the surface

        >>> viewport.surface.w = 640
        >>> viewport.surface.h = 480
        >>> viewport.surface_size_changed()

        >>> viewport.screen_pos(Vector(0, 0))
        (320, 240)

    """


def doctest_Viewport_keep_visible():
    """Tests for Viewport.keep_visible.

        >>> from world import Vector
        >>> from ui import Viewport
        >>> viewport = Viewport(SurfaceStub())

    Points that are already visible change nothing

        >>> viewport.keep_visible([Vector(0, 0)], 10)
        >>> viewport.keep_visible([Vector(100, 100)], 10)
        >>> viewport.keep_visible([Vector(-200, -100)], 10)
        >>> print viewport.origin, viewport.scale
        (0.000, 0.000) 1.0

    We can see the range of points that are inside the view margin, for
    a given margin size

        >>> viewport.world_inner_bounds(0)
        (-400.0, -300.0, 400.0, 300.0)
        >>> viewport.world_inner_bounds(10)
        (-390.0, -290.0, 390.0, 290.0)

    Points that are off-screen cause scrolling

        >>> viewport.keep_visible([Vector(600, 200)], 10)
        >>> print viewport.origin, viewport.scale
        (210.000, 0.000) 1.0

        >>> viewport.keep_visible([Vector(300, 700)], 10)
        >>> print viewport.origin, viewport.scale
        (210.000, 410.000) 1.0

        >>> viewport.keep_visible([Vector(-300, -100)], 10)
        >>> print viewport.origin, viewport.scale
        (90.000, 190.000) 1.0

        >>> viewport.world_inner_bounds(10)
        (-300.0, -100.0, 480.0, 480.0)

    If you specify several widely-separated points, they may cause a scale
    change

        >>> viewport.keep_visible([Vector(-300, -100),
        ...                        Vector(500, 400)], 10)

        >>> print viewport.origin, round(viewport.scale, 3)
        (99.732, 190.000) 0.974

        >>> xmin, ymin, xmax, ymax = viewport.world_inner_bounds(10)
        >>> xmin <= -300 and 500 <= xmax
        True
        >>> ymin <= -100 and 400 <= ymax
        True

    """


def doctest_FrameRateCounter_frame():
    """Tests for FrameRateCounter.frame

        >>> from ui import FrameRateCounter
        >>> frc = FrameRateCounter()

        >>> frc.frame()
        >>> len(frc.frames)
        1
        >>> frc.frame()
        >>> len(frc.frames)
        2

        >>> frc.frames = range(frc.avg_last_n_frames - 1)
        >>> frc.frame()
        >>> len(frc.frames) == frc.avg_last_n_frames
        True
        >>> frc.frame()
        >>> len(frc.frames) == frc.avg_last_n_frames
        True
        >>> frc.frame()
        >>> len(frc.frames) == frc.avg_last_n_frames
        True

        >>> frc.frames[:3]
        [2, 3, 4]

    """


def doctest_FrameRateCounter_reset():
    """Tests for FrameRateCounter.reset

        >>> from ui import FrameRateCounter
        >>> frc = FrameRateCounter()
        >>> frc.frames = range(15)
        >>> frc.reset()
        >>> frc.frames
        []

    """


def doctest_FrameRateCounter_fps():
    """Tests for FrameRateCounter.fps

        >>> from ui import FrameRateCounter
        >>> frc = FrameRateCounter()
        >>> frc.frames = []
        >>> frc.fps()
        0

        >>> frc.frames = [1000]
        >>> frc.fps()
        0

        >>> frc.frames += [1005]

    5 ms per frame corresponds to 200 frames per second.

        >>> frc.fps()
        200.0

        >>> frc.frames += [1020]

    The first frame took 5 ms, the next one 15 ms.  Average is 10 ms per frame,
    and this corresponds to 100 frames per second.

        >>> frc.fps()
        100.0

    """


def setUp(test=None):
    import pygame
    pygame.init() # so that pygame.key.name() works
    # unfortunately, on linux, if $DISPLAY is unset, pygame.init doesn't
    # complain, but pygame.key.name() returns 'unknown key' for all keys


def test_suite():
    path = os.path.join(os.path.dirname(__file__), os.path.pardir)
    if path not in sys.path:
        sys.path.append(path)
    return unittest.TestSuite([
                        doctest.DocTestSuite('ui', setUp=setUp),
                        doctest.DocTestSuite()])


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""
PySpaceWar game logic: creation of worlds, respawning of ships, synchronizing
with wall clock.

$Id: game.py 198 2006-01-14 20:44:33Z mg $
"""

import time
import random

from world import World, Vector, Ship, Planet


class PythonTimeSource(object):
    """A ticking clock based on time.time."""

    def __init__(self, ticks_per_second):
        self.ticks_per_second = ticks_per_second
        self.delta = 1.0 / ticks_per_second

    def now(self):
        """Return the current time."""
        return time.time()

    def wait(self, time_point):
        """Wait until now() becomes >= time_point.

        Returns True if we're on schedule; False if time_point was already
        past when ``wait`` was called.
        """
        time_to_sleep = time_point - self.now()
        if time_to_sleep > 0:
            time.sleep(time_to_sleep)
        return time_to_sleep >= 0


class Game(object):
    """A game.

    Example:

        g = Game.new(ships=2, planet_kinds=3)
        while True:
            g.wait_for_tick()
            # interact with game objects
            # draw the state of the game

    """

    TICKS_PER_SECOND = 20               # Number of game ticks per second
    DELTA_TIME = 2.0                    # World time units per game tick
    ROTATION_SPEED = 10 / DELTA_TIME    # Rotation angle per time unit
    FRONT_THRUST = 0.2 / DELTA_TIME     # Forward acceleration
    REAR_THRUST = 0.1 / DELTA_TIME      # Backward acceleration

    respawn_radius = 600                # Locale for respawning ships
    respawn_time = 100                  # Time before a dead ship respawns
    planet_placement_margin = 20        # Free space between planets
    ship_placement_margin = 100         # Free space between ships

    # Some debug information
    time_to_update = 0                  # Time to update game world
    time_waiting = 0                    # Time spent idling

    def __init__(self, rng=None):
        if rng is None:
            rng = random.Random()
        self.rng = rng
        self.world = World(rng)
        self.ships = []
        self.timers = {}
        self.time_source = PythonTimeSource(self.TICKS_PER_SECOND)
        self._next_tick = None
        self.controllers = []

    def randomly_position(self, obj, world_radius, margin=0):
        """Pick a random location for ``obj``."""
        obj.radius += margin
        while True:
            obj.position = Vector.from_polar(self.rng.uniform(0, 360),
                                             self.rng.uniform(0, world_radius))
            for other in self.world.objects:
                if other is not obj and self.world.collide(obj, other):
                    break
            else:
                break
            world_radius *= 1.1
        obj.radius -= margin

    def randomly_place(self, obj, world_radius, margin=0):
        """Place ``obj`` in a randomly chosen location."""
        self.randomly_position(obj, world_radius, margin)
        self.world.add(obj)
        if isinstance(obj, Ship):
            self.ships.append(obj)

    def respawn(self, ship):
        """Respawn a ship."""
        self.randomly_position(ship, self.respawn_radius,
                               self.ship_placement_margin)
        ship.velocity = Vector(0, 0)
        granularity = self.ROTATION_SPEED * self.DELTA_TIME
        ship.direction = self.rng.randrange(360 / granularity) * granularity
        ship.respawn()

    def auto_respawn(self):
        """Respawn dead ships after a timeout."""
        for ship in self.ships:
            if ship.dead:
                if ship not in self.timers:
                    self.timers[ship] = self.respawn_time
                else:
                    self.timers[ship] -= self.DELTA_TIME
                    if self.timers[ship] <= 0:
                        del self.timers[ship]
                        self.respawn(ship)

    def time_to_respawn(self, ship):
        """Return the time left before a dead ship will respawn.

        Returns 0 if the ship is not dead.
        """
        return self.timers.get(ship, 0)

    def skip_a_tick(self):
        """Skip a wall clock tick (the game is paused)."""
        if self._next_tick is None: # first time!
            self._next_tick = self.time_source.now() + self.time_source.delta
            self.time_waiting = 0
        else:
            start = time.time()
            self.time_source.wait(self._next_tick)
            self._next_tick = self.time_source.now() + self.time_source.delta
            self.time_waiting = time.time() - start

    def wait_for_tick(self):
        """Wait for the next game time tick."""
        if self._next_tick is None: # first time!
            self._next_tick = self.time_source.now() + self.time_source.delta
            self.time_waiting = 0
            on_schedule = True
        else:
            start = time.time()
            self.world.update(self.DELTA_TIME)
            self.auto_respawn()
            for controller in self.controllers:
                controller.control()
            self.time_to_update = time.time() - start
            start = time.time()
            on_schedule = self.time_source.wait(self._next_tick)
            self.time_waiting = time.time() - start
            self._next_tick += self.time_source.delta
        return on_schedule

    def new(cls, ships=2, planet_kinds=1, world_radius=1200,
            ship_start_radius=600, rng=None):
        """Create a new random game."""
        game = cls(rng)
        game.respawn_radius = ship_start_radius
        rng = game.rng
        n_planets = rng.randrange(2, 20)
        for n in range(n_planets):
            appearance = rng.randrange(planet_kinds)
            radius = rng.uniform(5, 40)
            mass = radius ** 3 * rng.uniform(2, 6)
            planet = Planet(appearance=appearance, radius=radius, mass=mass)
            game.randomly_place(planet, world_radius,
                                game.planet_placement_margin)
        for n in range(ships):
            granularity = cls.ROTATION_SPEED * cls.DELTA_TIME
            direction = rng.randrange(360 / granularity) * granularity
            ship = Ship(appearance=n, direction=direction)
            # Install a standard engine
            ship.rotation_speed = cls.ROTATION_SPEED
            ship.forward_power = cls.FRONT_THRUST
            ship.backward_power = cls.REAR_THRUST
            game.randomly_place(ship, ship_start_radius,
                                game.ship_placement_margin)
        return game

    new = classmethod(new)



#!/usr/bin/env python
"""
PySpaceWar benchmarks for optimisation work

$Id: benchmark.py 162 2006-01-08 14:20:10Z mg $
"""

import os
import sys
import time
import random
import optparse

import pygame
from pygame.locals import *


def setup_path():
    """Set up python path if running from a source tree."""
    pkgdir = os.path.join(os.path.dirname(__file__), 'src')
    print pkgdir
    if os.path.isdir(pkgdir):
        sys.path.insert(0, pkgdir)

setup_path()


from pyspacewar.game import Game
from pyspacewar.world import Ship
from pyspacewar.ai import AIController
from pyspacewar.ui import GameUI


def get_cpu_speed():
    try:
        rows = [row for row in file('/proc/cpuinfo')
                if row.startswith('cpu MHz')]
    except IOError:
        return 0
    try:
        return float(rows[0].split(':')[1])
    except IndexError:
        return 0


class DummyAIController(object):

    def __init__(self, ship):
        self.ship = ship

    def control(self):
        if self.ship.appearance % 2:
            self.ship.turn_left()
        else:
            self.ship.turn_right()
        if len(self.ship.world.objects) < 90:
            self.ship.launch()

class DummyTimeSource(object):

    delta = 0

    def now(self):
        return 0

    def wait(self, time):
        return True


class Stats(object):
    time = 0
    ticks = 0
    max_objects = 0
    min_objects = sys.maxint
    total_objects = 0
    best_time = sys.maxint
    worst_time = 0

    @property
    def avg_objects(self):
        if self.ticks == 0:
            return 0
        return self.total_objects / float(self.ticks)

    @property
    def ticks_per_second(self):
        if self.time == 0:
            return 0
        return self.ticks / float(self.time)

    @property
    def ms_per_tick(self):
        if self.ticks == 0:
            return 0
        return self.time * 1000.0 / self.ticks


class Benchmark(object):

    def __init__(self, seed=None, how_long=100,
                 ai_controller=DummyAIController, warmup_ticks=0,
                 profile=False, debug=False):
        self.seed = seed
        self.how_long = how_long
        self.ai_controller = ai_controller
        self.warmup_ticks = warmup_ticks
        self.profile = profile
        self.debug = debug

    def run(self):
        self.stats = Stats()
        self.init()
        self.stats.cpu_speed_before_warmup = get_cpu_speed()
        self.warmup()
        self.stats.cpu_speed_after_warmup = get_cpu_speed()
        if self.profile:
            from profile import Profile
            profiler = Profile()
            profiler.runcall(self.benchmark)
        else:
            self.benchmark()
        self.stats.cpu_speed_after_benchmark = get_cpu_speed()
        if self.profile:
            import pstats
            self.stats.profile_stats = pstats.Stats(profiler)
        return self.stats

    def warmup(self):
        tick = 0
        while tick < self.warmup_ticks:
            self.cycle()
            tick += 1

    def benchmark(self):
        game = self.game
        stats = self.stats
        start = now = time.time()
        while stats.ticks < self.how_long:
            prev = now
            self.cycle()
            now = time.time()
            stats.ticks += 1
            stats.max_objects = max(stats.max_objects, len(game.world.objects))
            stats.min_objects = min(stats.min_objects, len(game.world.objects))
            stats.total_objects += len(game.world.objects)
            stats.best_time = min(stats.best_time, now - prev)
            stats.worst_time = max(stats.worst_time, now - prev)
            stats.time = now - start


class LogicBenchmark(Benchmark):

    def init(self):
        game = Game.new(ships=2, rng=random.Random(self.seed))
        game.time_source = DummyTimeSource()
        ships = [obj for obj in game.world.objects if isinstance(obj, Ship)]
        game.controllers += map(self.ai_controller, ships)
        game.wait_for_tick() # first one does nothing serious
        self.game = game
        self.cycle = game.wait_for_tick


class GameBenchmark(Benchmark):

    def init(self):
        ui = GameUI()
        ui.rng = random.Random(self.seed)
        ui.show_debug_info = self.debug
        ui.init()
        for player_id, is_ai in enumerate(ui.ai_controlled):
            if is_ai:
                ui.toggle_ai(player_id)
        game = ui.game
        game.time_source = DummyTimeSource()
        game.controllers += map(self.ai_controller, ui.ships)
        game.wait_for_tick() # first one does nothing serious
        self.ui = ui
        self.game = game

    def cycle(self):
        self.ui.wait_for_tick()
        self.ui.draw()
        event = pygame.event.poll()
        if (event.type == QUIT or
            (event.type == KEYDOWN and event.key in (K_ESCAPE, K_q))):
            self.how_long = 0
            self.warmup_ticks = 0


def main():
    parser = optparse.OptionParser()
    parser.add_option('-s', '--seed', default=0,
                      help='specify random seed [default: %default]',
                      action='store', dest='seed', type='int')
    parser.add_option('-t', '--ticks', default=100,
                      help='specify number of game ticks [default: %default]',
                      action='store', dest='ticks', type='int')
    parser.add_option('-w', '--warmup', default=100,
                      help='warm up for a number of ticks [default: %default]',
                      action='store', dest='warmup', type='int')
    parser.add_option('-a', '--ai', default=DummyAIController,
                      help='use real AI logic [default: dumb logic]',
                      action='store_const', const=AIController,
                      dest='ai_controller')
    parser.add_option('-g', '--gui', default=LogicBenchmark,
                      help='benchmark drawing and logic [default: just logic]',
                      action='store_const', const=GameBenchmark,
                      dest='benchmark')
    parser.add_option('-d', '--debug', default=False,
                      help='show debug info during benchmark [default: %default]',
                      action='store_true', dest='debug')
    parser.add_option('-p', '--profile', default=False,
                      help='enable profiling [default: %default]',
                      action='store_true', dest='profile')
    parser.add_option('--psyco', default=False,
                      help='use Psyco [default: %default]',
                      action='store_true', dest='psyco')
    opts, args = parser.parse_args()
    print "=== Parameters ==="
    print
    if opts.psyco:
        try:
            import psyco
            psyco.full()
        except:
            print 'psyco not available'
        else:
            print 'using psyco'
    print 'random seed: %r' % opts.seed
    print 'warmup: %d' % opts.warmup
    print 'ticks: %d' % opts.ticks
    print 'ai: %s' % opts.ai_controller.__name__
    print 'benchmark: %s' % opts.benchmark.__name__
    print 'debug: %s' % opts.debug
    benchmark = opts.benchmark(opts.seed, opts.ticks, opts.ai_controller,
                               opts.warmup, opts.profile, opts.debug)
    start_time = time.time()
    stats = benchmark.run()
    total_time = time.time() - start_time
    print
    print "=== CPU ==="
    print
    print 'CPU speed before warmup: %.0f MHz' % stats.cpu_speed_before_warmup
    print 'CPU speed after warmup: %.0f MHz' % stats.cpu_speed_after_warmup
    print 'CPU speed after benchmark: %.0f MHz' % stats.cpu_speed_after_benchmark
    print
    print "=== Results ==="
    print
    print 'total time: %.3f seconds' % total_time
    print 'ticks: %d' % stats.ticks
    print 'objects: min=%d avg=%.1f max=%d' % (
                stats.min_objects,
                stats.avg_objects,
                stats.max_objects)
    print 'ticks per second: avg=%.3f' % stats.ticks_per_second
    print 'ms per tick: min=%.3f avg=%.3f max=%.3f' % (
                stats.best_time * 1000.0,
                stats.ms_per_tick,
                stats.worst_time * 1000.0)

    if opts.profile:
        stats = stats.profile_stats
        stats.strip_dirs()
        print
        print "== Stats by internal time ==="
        print
        stats.sort_stats('time', 'calls')
        stats.print_stats(40)
        print
        print "== Stats by number of calls, with callers ==="
        print
        stats.sort_stats('calls', 'time')
        stats.print_callers(20)
        print
        print "== Stats by cumulative time, with calees ==="
        print
        stats.sort_stats('cumulative', 'calls')
        stats.print_callees(20)

if __name__ == '__main__':
    main()



"""
The world of PySpaceWar

$Id: world.py 245 2007-01-05 01:25:05Z mg $
"""

import math
import random
import time


class Vector(tuple):
    """A two-dimensional real vector.

        >>> v1 = Vector(3.5, -2.5)
        >>> v1.x, v1.y
        (3.5, -2.5)

    Vectors are immutable.

        >>> v1.x = 5
        Traceback (most recent call last):
          ...
        AttributeError: can't set attribute

    """

    # Nice accessories.  Sort of expensive, though: according to timeit,
    # v.x is 5 times slower than v[0].  It would be possible to shave off
    # 2.5 milliseconds off world update time by using [0], [1] instead of
    # .x, .y everywhere.  Time to rewrite in C?
    x = property(lambda self: self[0])
    y = property(lambda self: self[1])

    def __new__(cls, x, y):
        return tuple.__new__(cls, (x, y))

    def from_polar(direction, magnitude=1.0):
        """Create a new vector from polar coordinates.

        ``direction`` is the angle in degrees (0 points in the direction of the
        x axis, 90 points in the direction of the y axis).

            >>> print Vector.from_polar(90, 3.5)
            (0.000, 3.500)

            >>> print Vector.from_polar(135, 1)
            (-0.707, 0.707)

        """
        angle = direction * math.pi / 180
        return Vector(magnitude * math.cos(angle), magnitude * math.sin(angle))
    from_polar = staticmethod(from_polar)

    def __str__(self):
        """Return an (approximate) human-readable string representation.

            >>> print Vector.from_polar(-90, 1)
            (0.000, -1.000)

        """
        return '(%.3f, %.3f)' % self

    def __repr__(self):
        """Return an accurate string representation.

            >>> Vector.from_polar(-90, 1)           # doctest:+ELLIPSIS
            Vector(6.123...e-17, -1.0)

        """
        return 'Vector(%r, %r)' % self

    def __mul__(self, factor):
        """Multiply the vector by a scalar.

            >>> Vector(1.5, 2.5) * 3
            Vector(4.5, 7.5)
            >>> 3 * Vector(1.5, 2.5)
            Vector(4.5, 7.5)

        """
        return Vector(self.x * factor, self.y * factor)

    __rmul__ = __mul__

    def dot_product(self, other):
        """Compute the dot product of two vectors.

            >>> Vector(3, 5).dot_product(Vector(-1, 2))
            7

        """
        return self.x * other.x + self.y * other.y

    def __div__(self, divisor):
        """Divide the vector by a scalar.

            >>> Vector(1.5, 7.5) / 3
            Vector(0.5, 2.5)

            >>> print Vector(1, 2) / 3
            (0.333, 0.667)

        """
        return Vector(self.x / float(divisor), self.y / float(divisor))

    def __add__(self, other):
        """Add two vectors.

            >>> Vector(1.5, 2.5) + Vector(-0.5, 3)
            Vector(1.0, 5.5)

        """
        return Vector(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        """Subtract two vectors.

            >>> Vector(1.5, 2.5) - Vector(-0.5, 3)
            Vector(2.0, -0.5)

        """
        return Vector(self[0] - other[0], self[1] - other[1])

    def __neg__(self):
        """Multiply the vector by -1.

            >>> -Vector(1.5, 2.5)
            Vector(-1.5, -2.5)

        """
        return Vector(-self.x, -self.y)

    def length(self):
        """Compute the length of the vector.

            >>> Vector(3, 4).length()
            5.0

        """
        return math.hypot(self[0], self[1])

    def direction(self):
        """Compute the direction of the vector (in degrees).

            >>> Vector(1, 0).direction()
            0.0
            >>> Vector(0, 1).direction()
            90.0
            >>> Vector(-1, 0).direction()
            180.0
            >>> Vector(0, -1).direction()
            270.0
            >>> Vector(-1, 1).direction()
            135.0

        """
        angle = math.atan2(self.y, self.x) * 180 / math.pi
        if angle < 0:
            angle += 360
        return angle

    def perpendicular(self):
        """Compute a perpendicular vector of the same length.

        Returns the vector rotated clockwise by 90 degrees (assuming the usual
        mathematical direction of axes: x points right, y points up).

            >>> print Vector(2, 1).perpendicular()
            (-1.000, 2.000)

        """
        return Vector(-self.y, self.x)

    def scaled(self, new_length=1.0):
        """Scale the vector to a given magnitude.

            >>> v = Vector(2, 1).scaled(5)
            >>> print v.length()
            5.0
            >>> print v
            (4.472, 2.236)

        If you omit the magnitude, you get a normalized vector

            >>> print v.scaled().length()
            1.0

        """
        return self * new_length / self.length()


class World(object):
    """The game universe.

    Example:

        w = World()
        w.add(planet)
        w.add(ship)
        while True:
            # Tell the ship what to do during the next update
            # Draw the state of the world
            w.update(0.1)
            time.sleep(0.1)

    """

    GRAVITY = 0.01          # constant of gravitation
    BOUNCE_SPEED_LOSS = 0.1 # lose 10% speed when bouncing off something

    # Some debug information
    time_for_gravitation = 0    # Time to calculate gravitation
    time_for_collisions = 0     # Time to detect collisions

    def __init__(self, rng=None):
        if rng is None:
            rng = random.Random()
        self.rng = rng
        self.time = 0.0
        self.objects = []
        self._objects_with_zero_radius = []
        self._objects_with_nonzero_radius = []
        self._in_update = False
        self._add_queue = []
        self._remove_queue = []

    def add(self, obj):
        """Add a new object to the universe.

        Objects living in the universe must have the following methods:

            gravitate(massive_object, delta_time)
                react to gravity from massive_object for a particular time
            move(delta_time)
                move for a particular time
            collission(other_object)
                handle a collision
            distanceTo(other_object)
                calculate the distance to another object

        and a ``radius`` attribute, used for collision detection.

        Objects added to a universe will get a ``world`` attribute.
        """
        if self._in_update:
            if obj in self._remove_queue:
                self._remove_queue.remove(obj)
            else:
                self._add_queue.append(obj)
        else:
            self.objects.append(obj)
            obj.world = self
            if obj.radius:
                self._objects_with_nonzero_radius.append(obj)
            else:
                self._objects_with_zero_radius.append(obj)

    def remove(self, obj):
        """Remove an object from the universe."""
        if self._in_update:
            if obj in self._add_queue:
                self._add_queue.remove(obj)
            else:
                self._remove_queue.append(obj)
        else:
            self.objects.remove(obj)
            obj.world = None
            if obj.radius:
                self._objects_with_nonzero_radius.remove(obj)
            else:
                self._objects_with_zero_radius.remove(obj)

    def update(self, dt):
        """Make time happen (dt time units of it).

        Calculates gravity effects, moves the objects around, checks for
        collisions.
        """
        self._in_update = True
        self.time += dt
        # Gravity: affects velocities, but not positions
        start = time.time()
        for massive_obj in self.objects:
            if not massive_obj.mass:
                continue
            for obj in self.objects:
                if obj is not massive_obj:
                    obj.gravitate(massive_obj, dt)
        self.time_for_gravitation = time.time() - start
        # Movement: affects positions, may affect velocities
        for obj in self.objects:
            obj.move(dt)
        # Collision detection: may affect positions and velocities
        start = time.time()
        for n, obj1 in enumerate(self._objects_with_nonzero_radius):
            for obj2 in (self._objects_with_nonzero_radius[n+1:] +
                         self._objects_with_zero_radius):
                if self.collide(obj1, obj2):
                    obj1.collision(obj2)
                    obj2.collision(obj1)
        self.time_for_collisions = time.time() - start
        self._in_update = False
        if self._add_queue:
            for obj in self._add_queue:
                self.add(obj)
            self._add_queue = []
        if self._remove_queue:
            for obj in self._remove_queue:
                self.remove(obj)
            self._remove_queue = []

    def collide(self, obj1, obj2):
        """Check whether two objects collide."""
        collision_distance = obj1.radius + obj2.radius
        return obj1.distanceTo(obj2) < collision_distance


class Object(object):
    """A material object in the game universe.

        >>> o = Object(Vector(45.0, 110.0), appearance=42)
        >>> o.position
        Vector(45.0, 110.0)
        >>> o.velocity
        Vector(0.0, 0.0)
        >>> o.mass
        0
        >>> o.radius
        0

    Appearance is an opaque field that user interface code can use to
    distinguish between different objects of the same kind.

        >>> o.appearance
        42

    """

    def __init__(self, position=Vector(0.0, 0.0), velocity=Vector(0.0, 0.0),
                 mass=0, radius=0, appearance=0):
        self.position = position
        self.mass = mass
        self.radius = radius
        self.velocity = velocity
        self.appearance = appearance
        self.world = None
        self.bounce_effect = None

    def distanceTo(self, other):
        """Calculate the distance to another object.

            >>> sun = Object(Vector(30, 40))
            >>> tincan = Object(Vector(0, 0))
            >>> tincan.distanceTo(sun)
            50.0
            >>> sun.distanceTo(tincan)
            50.0

        """
        sp = self.position
        op = other.position
        return math.hypot(sp[0] - op[0], sp[1] - op[1])

    def gravitate(self, massive_object, dt):
        """React to gravity from massive_object for a particular time.

            >>> sun = Object(Vector(0, 10), mass=200)
            >>> tincan = Object(Vector(0, 0))
            >>> tincan.world = World()
            >>> tincan.gravitate(sun, 1.0)
            >>> print tincan.velocity
            (0.000, 0.020)

            >>> tincan.gravitate(sun, 1.0)
            >>> print tincan.velocity
            (0.000, 0.040)

            >>> tincan.gravitate(sun, .5)
            >>> print tincan.velocity
            (0.000, 0.050)

        """
        # Newton's laws of motion:
        #   F = G * m1 * m2 / r**2
        #   a = F / m1 = G * m2 / r ** 2
        # where
        #   F is force of attraction
        #   G is the constant of gravitation
        #   m1 and m2 are the masses of interacting bodies
        #   r is the distance between interacting bodies
        #   a is the acceleration of the body with mass m1
        # We consider time t from t0 to t1 where t1 = t0 + dt
        #   v(t1) = v(t0) + integral(t=t0..t1, a(t)dt)
        #   a(t) = G * m2 / r(t) ** 2
        # For simplicity's sake let's assume r(t) is constant.  Then a(t) is
        # also constant, and
        #   v(t1) = v(t0) + a * dt

        # Nice code:
        #   vector = massive_object.position - self.position
        #   distance = vector.length()
        #   magnitude = self.world.GRAVITY * massive_object.mass / distance ** 2
        #   acceleration = vector * (magnitude * dt / distance)
        #   self.velocity += acceleration
        # The equivalent fast code:
        dx = massive_object.position[0] - self.position[0]
        dy = massive_object.position[1] - self.position[1]
        distance = math.hypot(dx, dy)
        f = self.world.GRAVITY * massive_object.mass * dt / distance ** 3
        self.velocity = Vector(self.velocity[0] + dx * f,
                               self.velocity[1] + dy * f)

    def move(self, dt):
        """Move for a particular time.

            >>> tincan = Object(Vector(0, 0), velocity=Vector(0.5, 1.0))
            >>> tincan.move(1.0)
            >>> tincan.position
            Vector(0.5, 1.0)

            >>> tincan.move(1.0)
            >>> tincan.position
            Vector(1.0, 2.0)

            >>> tincan.move(2.0)
            >>> tincan.position
            Vector(2.0, 4.0)

        """
        self.position += self.velocity * dt

    def collision(self, other):
        """Handle a collision with another object.

        The default implementation is bouncing off things.
        """
        self.bounce(other)

    def bounce(self, other):
        """Bounce from another object.

            >>> asteroid = Object(Vector(3, 4), velocity=Vector(0.1, 0.2),
            ...                   mass=14, radius=2)
            >>> tincan = Object(Vector(0.5, 4), velocity=Vector(1.5, -0.3),
            ...                 mass=1, radius=1)
            >>> tincan.world = World()

            >>> tincan.bounce(asteroid)
            >>> print tincan.velocity
            (-1.350, -0.270)
            >>> print tincan.position
            (0.000, 4.000)

        The bounce is not physically realistic (e.g. total energy/momentum
        is not preserved).
        """
        normal = (self.position - other.position).scaled()
        delta = normal.x * self.velocity.x + normal.y * self.velocity.y
        self.velocity -= normal.scaled(2 * delta)
        # Let's lose some speed
        self.velocity *= 1 - self.world.BOUNCE_SPEED_LOSS
        # Let's also make sure the objects do not overlap
        collision_distance = other.radius + self.radius
        self.position = other.position + normal.scaled(collision_distance)
        if self.bounce_effect:
            self.bounce_effect(self, other)

    def add_debris(self, howmany=None, maxdistance=1.0, time=5.0):
        """Add some debris."""
        rng = self.world.rng
        if not howmany:
            howmany = rng.randrange(3, 6)
        for n in range(howmany):
            color = (rng.randrange(0xf0, 0xff),
                     rng.randrange(0x70, 0x90),
                     rng.randrange(0, 0x20))
            velocity = self.velocity * 0.3
            velocity += Vector.from_polar(rng.uniform(0, 360),
                                          rng.uniform(0, maxdistance))
            debris = Debris(self.position, velocity, appearance=color,
                            time_limit=time)
            self.world.add(debris)


class Planet(Object):
    """A planet in the game universe.

    Planets have mass and attract other things.  Planets themselves are not
    affected by gravity (computing stable orbits for N planets is too
    difficult, especially with the numeric approximations I take).
    """

    def gravitate(self, massive_object, dt):
        """Don't react to gravity."""
        pass

    def collision(self, other):
        """Handle a collision: nothing happens to a planet."""
        pass


class Ship(Object):
    """A powered ship.

        >>> ship = Ship(Vector(0, 0), size=10, direction=45)
        >>> ship.size
        10
        >>> ship.radius
        6.0

        >>> ship.health
        1.0

    You tell the ship what to do, and the ship does it

        >>> ship.turn_left()
        >>> ship.accelerate()
        >>> ship.move(1.0)
        >>> ship.direction
        50.0
        >>> print ship.velocity.length()
        0.1

    """

    SIZE_TO_RADIUS = 0.6    # Ships aren't circular.  To simulate more or
                            # less convincing collisions we need to have a
                            # collision radius smaller than ship size.

    forward_power = 0.1     # Default engine power for forward thrust
    backward_power = 0.05   # Default engine power for backward thrust
    brake_factor = 0.95     # Default brake effectiveness (lose 5% speed)
    brake_threshold = 0.5   # Speed below which brakes are 100% efficient
    rotation_speed = 5      # Lateral thruster power (angles per time unit)
    launch_speed = 3.0      # Missile launch speed
    missile_recoil = 0.01   # Missile recoil factor
    missile_damage = 0.6    # Damage done by the missile
    collision_damage = 0.05 # Damage done by a collision
    missile_time_limit = (1200, 1300) # Range for missile self-destruct timer

    def __init__(self, position=Vector(0, 0), velocity=Vector(0, 0), size=10,
                 direction=0, appearance=0):
        Object.__init__(self, position, velocity=velocity,
                        radius=size * self.SIZE_TO_RADIUS,
                        appearance=appearance)
        self.size = size
        self.direction = direction
        self.forward_thrust = 0
        self.rear_thrust = 0
        self.left_thrust = 0
        self.right_thrust = 0
        self.engage_brakes = False
        self.health = 1.0
        self.frags = 0
        self.dead = False
        self.spawn_time = 0 # the value of world.time when last respawned
        self.launch_effect = None
        self.hit_effect = None
        self.explode_effect = None
        self.respawn_effect = None

    def _set_direction(self, direction):
        """Set the direction of the ship.

        The direction is given in angles.  It will be normalized so that
        0 <= direction < 360.

        The direction_vector attribute is also set.
        """
        direction = direction % 360
        if direction < 0: direction += 360
        self._direction = direction
        self.direction_vector = Vector.from_polar(direction)

    direction = property(lambda self: self._direction, _set_direction)

    def turn_left(self):
        """Tell the ship to turn left."""
        if self.dead:
            return
        self.left_thrust = self.rotation_speed

    def turn_right(self):
        """Tell the ship to turn right."""
        if self.dead:
            return
        self.right_thrust = self.rotation_speed

    def accelerate(self):
        """Tell the ship to accelerate in the direction of the ship."""
        if self.dead:
            return
        self.forward_thrust = self.forward_power

    def backwards(self):
        """Tell the ship to accelerate in the opposite direction."""
        if self.dead:
            return
        self.rear_thrust = self.backward_power

    def brake(self):
        """Tell the ship to brake."""
        if self.dead:
            return
        if self.velocity != (0, 0):
            self.engage_brakes = True

    def gravitate(self, massive_object, dt):
        """Don't react to gravity.  Because of, um, anti-gravity engines."""
        if not self.dead:
            return
        Object.gravitate(self, massive_object, dt)

    def move(self, dt):
        """Apply thrusters and move in the universe."""
        if self.left_thrust:
            self.direction += self.left_thrust * dt
            self.left_thrust = 0
        if self.right_thrust:
            self.direction -= self.right_thrust * dt
            self.right_thrust = 0
        if self.forward_thrust:
            self.velocity += self.direction_vector * self.forward_thrust * dt
            self.forward_thrust = 0
        if self.rear_thrust:
            self.velocity -= self.direction_vector * self.rear_thrust * dt
            self.rear_thrust = 0
        if self.engage_brakes:
            if self.velocity.length() <= self.brake_threshold:
                self.velocity = Vector(0.0, 0.0)
            else:
                self.velocity *= self.brake_factor
            self.engage_brakes = False
        Object.move(self, dt)

    def collision(self, other):
        """Handle a collision."""
        killed_by = None
        if isinstance(other, Debris):
            return
        elif isinstance(other, Missile):
            self.health -= self.missile_damage
            killed_by = other.launched_by
            self.velocity += other.velocity * self.missile_recoil
            if self.hit_effect:
                self.hit_effect(self, other)
        else:
            self.health -= self.collision_damage
            self.bounce(other)
        if self.health < 0 and not self.dead:
            self.die(killed_by)

    def die(self, killed_by=None):
        """The ship has received terminal damage."""
        self.dead = True
        self.forward_thrust = 0
        self.rear_thrust = 0
        self.left_thrust = 0
        self.right_thrust = 0
        if killed_by is None or killed_by is self:
            self.frags -= 1
        else:
            killed_by.frags += 1
        self.add_debris(time=50, maxdistance=self.size * 0.5,
                        howmany=self.world.rng.randrange(9, 21))
        if self.explode_effect:
            self.explode_effect(self, killed_by)

    def respawn(self):
        """Respawn back into the world."""
        self.dead = False
        self.health = 1.0
        if self.world:
            self.spawn_time = self.world.time
        if self.respawn_effect:
            self.respawn_effect(self)

    def launch(self):
        """Launch a missile."""
        if self.dead:
            return
        direction_vector = self.direction_vector
        time_limit = self.world.rng.uniform(*self.missile_time_limit)
        missile = Missile(self.position + direction_vector * self.size,
                          self.velocity + direction_vector * self.launch_speed,
                          self.appearance, launched_by=self,
                          time_limit=time_limit)
        recoil = direction_vector * self.launch_speed * self.missile_recoil
        self.velocity -= recoil
        self.world.add(missile)
        if self.launch_effect:
            self.launch_effect(self, missile)


class Missile(Object):
    """A missile.

    Ships fire missiles.  Missiles are unpowered and cannot manoeuvre.
    Missiles explode on contact and self-destruct after a set time limit.
    """

    def __init__(self, position=Vector(0, 0), velocity=Vector(0, 0),
                 appearance=0, launched_by=None, time_limit=1e1000):
        Object.__init__(self, position=position, velocity=velocity,
                        appearance=appearance)
        self.launched_by = launched_by
        self.time_limit = time_limit
        self.dead = False

    def move(self, dt):
        """Move in the universe.  Check the self-destruct timer."""
        Object.move(self, dt)
        self.time_limit -= dt
        if self.time_limit < 0:
            self.explode()

    def explode(self):
        """Self-destruct."""
        if not self.dead:
            self.dead = True
            self.add_debris()
            self.world.remove(self)

    def collision(self, other):
        """Explode on collision."""
        self.explode()


class Debris(Object):
    """Debris is what remains when something explodes."""

    def __init__(self, position=Vector(0, 0), velocity=Vector(0, 0),
                 appearance=0, time_limit=10):
        Object.__init__(self, position, velocity=velocity,
                        appearance=appearance)
        self.time_limit = time_limit

    def move(self, dt):
        """Move in the universe.  Check the time left to live."""
        Object.move(self, dt)
        self.time_limit -= dt
        if self.time_limit < 0:
            self.world.remove(self)




"""
PySpaceWar command line processing and main loop

$Id: main.py 260 2007-01-14 20:03:44Z mg $
"""

import sys
import optparse

from pyspacewar.ui import GameUI


def use_psyco():
    """Use Psyco (if available) to speed things up."""
    try:
        import psyco
        psyco.full()
    except ImportError:
        pass


def main():
    """Run PySpaceWar."""
    use_psyco()
    ui = GameUI()
    ui.load_settings()
    parser = optparse.OptionParser()
    parser.add_option('-f', '--fullscreen', default=None,
                      help='start in full-screen mode',
                      action='store_true', dest='fullscreen')
    parser.add_option('-w', '--windowed',
                      help='start in windowed mode',
                      action='store_false', dest='fullscreen')
    parser.add_option('--sound', default=None,
                      help='enable sound',
                      action='store_true', dest='sound')
    parser.add_option('--no-sound',
                      help='disable sound',
                      action='store_false', dest='sound')
    parser.add_option('--music', default=None,
                      help='enable music',
                      action='store_true', dest='music')
    parser.add_option('--no-music',
                      help='disable music',
                      action='store_false', dest='music')
    parser.add_option('-d', '--debug', default=False,
                      help='show debug timings',
                      action='store_true', dest='debug')
    parser.add_option('-m', '--mode', default=None, metavar='WxH',
                      help='video mode for fullscreen (e.g. -m 640x480);'
                           ' note that in windowed mode the window size be 20%'
                           ' smaller',
                      action='store', dest='mode')
    opts, args = parser.parse_args()
    if opts.fullscreen is not None:
        ui.fullscreen = opts.fullscreen
    if opts.debug is not None:
        ui.show_debug_info = opts.debug
    if opts.sound is not None:
        ui.sound = opts.sound
    if opts.music is not None:
        ui.music = opts.music
    if opts.mode:
        try:
            w, h = opts.mode.split('x')
            ui.fullscreen_mode = int(w), int(h)
        except ValueError:
            print >> sys.stderr, ('pyspacewar: error: invalid mode: %s'
                                  % opts.mode)
            sys.exit(1)
    ui.init()
    try:
        while True:
            ui.wait_for_tick()
            ui.interact()
            ui.draw()
    except (KeyboardInterrupt, SystemExit):
        ui.save_settings()

